const { detectDeadCode, disabledFolders } = require('./detectDeadCode');
// eslint-disable-next-line import/extensions
const { Env } = require('../types');
const defaultOptions = {
  patterns: [`!(${disabledFolders.join('|')})/**/*.*`],
  exclude: [],
  failOnHint: false,
  detectUnusedFiles: true,
  detectUnusedExport: true,
};

class DetectDeadCodePlugin {
  options = defaultOptions;

  constructor(options) {
    if (!options) {
      return;
    }

    this.options = {
      ...this.options,
      ...options,
    };
  }

  apply(compiler) {
    if (!this.options.context) {
      this.options = {
        ...this.options,
        context: compiler.context,
      };
    }

    compiler.hooks.afterEmit.tapAsync('DetectDeadCodePlugin', this.handleAfterEmit);
  }

  handleAfterEmit = (compilation, callback) => {
    detectDeadCode(compilation, this.options);
    callback();
  };
}

module.exports = async function addDetectDeadCodePlugin(opts) {
  const { config, userConfig } = opts;
  const isDev = opts.env === Env.development;

  if (userConfig.deadCode && !isDev) {
    config.plugin('detect-dead-code-plugin').use(DetectDeadCodePlugin, [userConfig.deadCode]);
  }
};
