import { UserLayout } from '@wisdom-utils/components/lib/AppLayout';
import LoadingComponent from '@wisdom-utils/components/lib/AppLayout/components/PageLoading';
import { dynamic } from '@wisdom-utils/runtime';

import BasicLayout from '../layouts/BasicLayout';
import BootPage from '../pages/bootpage';
import UsingAnalysis from '../pages/cloud/analysis/using';
import CommonMenu from '../pages/commonMenu';
import Iframe from '../pages/iframe';
import Login from '../pages/user/login';
import NoSecret from '../pages/user/login/noSecret';
import Notifications from '../pages/system/notifications';

export const dyRoutes = (routes, layout, theme) => {
  // eslint-disable-next-line no-shadow
  const dyRoutes = routes || [];
  return {
    routes: [
      {
        path: '/user',
        component: UserLayout,
        beforeEnter(props) {
          console.log(props);
        },
        routes: [
          {
            name: 'login',
            path: '/user/login',
            component: Login,
            props: {
              onEnter(props) {
                console.log("login props", props)
              }
            }
          },
          {
            name: 'noscret',
            path: '/user/noscret',
            component: NoSecret,
          },
        ],
      },
      {
        path: '/industry',
        component: BootPage,
      },
      {
        path: '/cloud/analysis/using',
        component: UsingAnalysis,
      },
      {
        path: '/',
        // component: theme === 'lightgreen.css' ? HNLayout : BasicLayout,
        component: BasicLayout,
        routes: [
          ...dyRoutes,
          {
            path: '/commonmenu',
            component: CommonMenu,
            // name: '菜单收藏'
          },
          { 
            // 历史消息箱
            path: '/system/notifications',
            component: Notifications,
          },
          {
            path: '/iframe',
            component: Iframe,
          },
          {
            path: '/404',
            component: dynamic({
              loader: () => import(/* webpackChunkName: 'p__404' */ '../pages/exception/404'),
              loading: LoadingComponent,
            }),
          },
          {
            path: '/500',
            component: dynamic({
              loader: () => import(/* webpackChunkName: 'p__500' */ '../pages/exception/500'),
              loading: LoadingComponent,
            }),
          },
        ],
      },

      process.env.NODE_ENV === 'development' && {
        path: '/openapi',
        exact: true,
        component: dynamic({
          loader: () =>
            import(
              /* webpackChunkName: 'civ__plugin-openapi__openapi' */ '@wisdom-utils/components/lib/AppLayout/openapi/index'
            ),
          loading: LoadingComponent,
        }),
      },
      {
        path: '/404',
        component: dynamic({
          loader: () => import(/* webpackChunkName: 'p__404' */ '../pages/exception/404'),
          loading: LoadingComponent,
        }),
      },
      {
        path: '/500',
        component: dynamic({
          loader: () => import(/* webpackChunkName: 'p__500' */ '../pages/exception/500'),
          loading: LoadingComponent,
        }),
      },
    ],
  };
};
